import argparse
import os
import pickle


# ---------------------------------------------------
# Update and Create a pickle file for each module in Solutions Folder
# ---------------------------------------------------
def funcOfSolutions(solutionsfilelist, destPath, pathToEmptyDict, dictFile):
    for eachPath in solutionsfilelist:
        splittedPath = eachPath.split("/")

        rootFolder = splittedPath[-3]
        parentFolder = splittedPath[-2]
        filename = os.path.splitext(splittedPath[-1])[0]

        with open(pathToEmptyDict, "rb") as inF:
            emptyDict = pickle.load(inF)

        with open(eachPath, "rb") as inF2:
            treeNodes = pickle.load(inF2)

        output_log = f"Creating bigram for file: {filename}"

        for i in range(len(treeNodes) - 1):
            current_node = treeNodes[i]
            next_node = treeNodes[i + 1]

            if current_node not in dictFile or next_node not in dictFile:
                output_log += f"\nSkipping pair: {current_node}, {next_node} as either or both nodes are not in dictFile."
                continue

            index = dictFile[current_node] + "$$" + dictFile[next_node]
            emptyDict[index] += 1.0
        
        output_log += "\n\n\n"


        dictionarypath = os.path.join(destPath, "Bigram")
        # filename = filename + ".pickle"
        filenamepath = os.path.join(dictionarypath, filename)

        try:
            os.makedirs(dictionarypath, exist_ok=True)
            with open(filenamepath, "wb") as outF:
                pickle.dump(emptyDict, outF)
        except OSError as error:
            print("unable to create")

        try:
            os.makedirs(os.path.join(destPath, "logs"), exist_ok=True)
            with open(os.path.join(destPath, "logs", "bigram.log"), "a") as logF:
                logF.write(output_log)
        except FileNotFoundError:
            with open(os.path.join(destPath, "logs" "bigram.log"), "w") as logF:
                logF.write(output_log)
        



def main():
    # ---------------------------------------------------
    # ---------------------------------------------------
    # Put the path for the Empty Bigram Dictionary file
    # ---------------------------------------------------
    # ---------------------------------------------------
    current_script_path = os.path.abspath(__file__)

    # Get the directory containing the current script
    current_script_directory = os.path.dirname(current_script_path)
    relative_pathToEmptyDict = "../Processed/EmptyDictionaries/dictOfBigram.pickle"

    absolute_pathToEmptyDict = os.path.join(current_script_directory, relative_pathToEmptyDict)

    # ---------------------------------------------------
    # ---------------------------------------------------
    # Put the path for the Empty Bigram Dictionary file
    # ---------------------------------------------------
    # ---------------------------------------------------

    # ---------------------------------------------------
    # ---------------------------------------------------
    # Put the path for DictOfNodes.py file
    # ---------------------------------------------------
    # ---------------------------------------------------
    relative_pathToDictOfNodes = "../Processed/DictOfNodes/DictOfNodes.pickle"

    with open(
        os.path.join(current_script_directory, relative_pathToDictOfNodes), "rb"
    ) as inF:
        dictFile = pickle.load(inF)

    # ---------------------------------------------------
    # ---------------------------------------------------
    # Put the path for DictOfNodes.py file
    # ---------------------------------------------------
    # ---------------------------------------------------

    # ---------------------------------------------------
    # Take input from user for each function pickle file and DictOfNodes.pickle
    # ---------------------------------------------------

    cmdparser = argparse.ArgumentParser()
    cmdparser.add_argument(
        "parent",
        help="Add path for the folder that contains pickle files for each function in two folders Control & Autopilot",
    )
    cmdparser.add_argument("destination")

    args = cmdparser.parse_args()

    parentFolder = args.parent
    destPath = args.destination

    # print("parentFolder: ", parentFolder)
    # print("destPath: ", destPath)

    # ---------------------------------------------------
    # Define paths for Control and Autopilot folder
    # ---------------------------------------------------

    # ControlFolder = os.path.join(parentFolder, "Control")
    # AutoFolder = os.path.join(parentFolder, "Autopilot")
    SolutionsFolder = parentFolder

    # print("SolutionsFolder: ", SolutionsFolder)

    # ---------------------------------------------------
    # Create links for each file within each subfolder (Control & Autopilot)
    # ---------------------------------------------------

    # controlfolderlist = []
    # autopilotfolderlist = []

    # controlfilelist = []
    # autopilotfilelist = []

    solutionsfilelist = []

    # for folders in os.listdir(ControlFolder):
    #     controlfolderlist.append(os.path.join(ControlFolder, folders))

    # for eachparentfolder in controlfolderlist:
    #     for root, dirs, files in os.walk(eachparentfolder):
    #         for eachfile in os.listdir(root):
    #             controlfilelist.append(os.path.join(root, eachfile))

    # for folders in os.listdir(AutoFolder):
    #     autopilotfolderlist.append(os.path.join(AutoFolder, folders))

    # for eachparentfolder in autopilotfolderlist:
    #     for root, dirs, files in os.walk(eachparentfolder):
    #         for eachfile in os.listdir(root):
    #             autopilotfilelist.append(os.path.join(root, eachfile))

    
    for file in os.listdir(SolutionsFolder):
        if file.endswith(".pickle"):
            solutionsfilelist.append(os.path.join(SolutionsFolder, file))
    

    # ---------------------------------------------------
    # Update and Create a pickle file for each function in Control Folder
    # ---------------------------------------------------

    # funcOfControl(controlfilelist, destPath, absolute_pathToEmptyDict, dictFile)

    # ---------------------------------------------------
    # Update and Create a pickle file for each function in Autopilot Folder
    # ---------------------------------------------------

    # funcOfAutopilot(autopilotfilelist, destPath, absolute_pathToEmptyDict, dictFile)

    # ---------------------------------------------------
    # Update and Create a pickle file for each function in Solutions Folder
    # ---------------------------------------------------

    funcOfSolutions(solutionsfilelist, destPath, absolute_pathToEmptyDict, dictFile)


if __name__ == "__main__":
    main()